import pytest
from src.matrix import Matrix

def test_matrix_bad_values():
    with pytest.raises(ValueError):
        mat = Matrix([[1, 2, 7, 8], [3, 4], [5, 6]])



def test_matrix_definition():
    mat = Matrix([[1, 2], [3, 4]])
    
    assert mat.num_rows() == 2
    assert mat.num_cols() == 2
    
    assert mat.get_value(0, 0) == 1
    assert mat.get_value(0, 1) == 2
    assert mat.get_value(1, 0) == 3
    assert mat.get_value(1, 1) == 4




def test_add_matrix():
    mat1 = Matrix([[1, 2], [3, 4]])
    mat2 = Matrix([[5, 6], [7, 8]])
    
    mat3 = mat1 + mat2
    
    assert mat3 == Matrix([[6, 8], [10, 12]])



def test_add_matrix_incompatible_size():
    mat1 = Matrix([[1, 2], [3, 4]])
    mat2 = Matrix([[5, 6]])
    
    with pytest.raises(ValueError):
        mat3 = mat1 + mat2



def test_multiply_matrix():
    mat1 = Matrix([[1, 2], [3, 4]])
    mat2 = Matrix([[5, 6], [7, 8]])
    
    mat3 = mat1 * mat2
    
    assert mat3 == Matrix([[19, 22], [43, 50]])


def test_multiply_matrix_incompatible_size():
    mat1 = Matrix([[1, 2], [3, 4]])
    mat2 = Matrix([[5, 6]])
    
    with pytest.raises(ValueError):
        mat3 = mat1 * mat2




def test_matrix_determinant():
    mat = Matrix([[1, 2], [3, 4]])
    
    det = mat.determinant()
    
    assert det == -2



def test_matrix_hadamard_product():
    mat1 = Matrix([[1, 2], [3, 4]])
    mat2 = Matrix([[5, 6], [7, 8]])
    
    hadamard_product = mat1.hadamard_product(mat2)
    
    assert hadamard_product == Matrix([[5, 12], [21, 32]])


def test_matrix_transpose():
    mat = Matrix([[1, 2], [3, 4]])
    
    transpose = mat.transpose()
    
    assert transpose == Matrix([[1, 3], [2, 4]])

def test_matrix_inverse():
    mat = Matrix([[1, 2], [3, 4]])
    
    inverse = mat.inverse()
    
    assert inverse == Matrix([[-2, 1], [1.5, -0.5]])

def test_matrix_singular():
    mat = Matrix([[1, 2], [2, 4]])
    
    with pytest.raises(ValueError):
        mat.inverse()



def test_matrix_wrong_dimension():
    mat = Matrix([[1, 2], [3, 4], [5, 6]])
    
    with pytest.raises(ValueError):
        mat.inverse()